import React, { useState } from 'react'

import { documentToReactComponents } from '@contentful/rich-text-react-renderer'
import renderRichText from './renderRichText'
import { FormProvider, useForm } from 'react-hook-form'
import axios from 'axios'

import {
  Banner,
  CallToAction,
  Callout,
  Card,
  CardWithExtended,
  ChakraAwesome,
  DActions,
  DFormSchema,
  Embed,
  Feature,
  Hero,
  Statistic,
  HomepageHero,
  Section,
  ExtendedSectionLeft,
  ExtendedSectionRight,
  SlantedSectionTop,
  Image,
  LatestNews,
} from 'law-centres-design-system'

import {
  Box,
  Button,
  Container,
  Heading,
  Link,
  SimpleGrid,
  Text,
  Stack,
  Flex,
  FormControl,
  FormLabel,
  Input,
} from '@chakra-ui/react'

const ContentTypes = {
  webform: (content, context, id) => {
    const methods = useForm()
    const [confirmation, setConfirmation] = useState()

    const onSubmit = (values) => {
      let data = Object.assign(values, { formId: id })

      axios.post('/api/forms/', data).then((res) => {
        if (res.data?.form?.confirmationMessage) {
          setConfirmation(res.data.form.confirmationMessage)
        } else {
          setConfirmation(
            'Sorry, something went wrong, please try again or contact support@lawcentres.org.uk'
          )
        }
      })
    }

    return (
      <Box maxW="xl" px="0" mb="5">
        {methods.formState.isSubmitSuccessful && (
          <>
            <Box textAlign="left">
              {confirmation && (
                <>
                  <Box mb="5">
                    <Heading color="gray.700" mb="5" size="md" fontWeight="800">
                      Form submitted
                    </Heading>

                    <ReactMarkdown // eslint-disable-line
                      components={{
                        a: (
                          { node, ...props } // eslint-disable-line
                        ) => (
                          <Link color="brand.Green" {...props}>
                            {props.children}
                          </Link>
                        ),
                        p: (
                          { node, ...props } // eslint-disable-line
                        ) => (
                          <Text color="gray.700" mb="3" {...props}>
                            {props.children}
                          </Text>
                        ),
                      }}
                    >
                      children={confirmation}
                    </ReactMarkdown>
                  </Box>
                  <Button
                    variant="simple"
                    leftIcon={<ChakraAwesome icon="arrow-left" />}
                    // eslint-disable-next-line
                    onClick={(e) => {
                      methods.reset()
                      setConfirmation('')
                    }}
                  >
                    Back
                  </Button>
                </>
              )}
              {!confirmation && (
                <>
                  <Heading color="gray.700" mb="3" size="md" fontWeight="800">
                    Sending...
                  </Heading>
                  <ChakraAwesome
                    mb="0"
                    color="brand.Green"
                    size="3x"
                    spin
                    icon="spinner"
                  />
                </>
              )}
            </Box>
          </>
        )}
        {!methods.formState.isSubmitSuccessful && (
          <>
            <Heading color="gray.700" mb="7" size="md" fontWeight="800">
              {content.title}
            </Heading>
            <FormProvider {...methods}>
              <form onSubmit={methods.handleSubmit(onSubmit)}>
                <Box mb="5">
                  <DFormSchema useForm={methods} schema={content.fields} />
                </Box>
                <DActions />
              </form>
            </FormProvider>
          </>
        )}
      </Box>
    )
  },
  componentgrid: (content) => (
    <SimpleGrid
      columns={[1, null, `${content.columns}`]}
      autoRows="max-content"
      spacing="10"
    >
      <>
        {content.components.map((component, key) => (
          <Box key={key}>{renderComponent(component)}</Box>
        ))}
      </>
    </SimpleGrid>
  ),

  componentcontainer: (content) => {
    const isColumn = content.orientation === 'column'
    const isRow = content.orientation === 'row'
    const isLeftAlignment = content.alignment === 'start'
    const isRightAlignment = content.alignment === 'end'

    const alignItemsValue = isColumn
      ? isLeftAlignment
        ? 'flex-start'
        : isRightAlignment
        ? 'flex-end'
        : 'center'
      : 'flex-start'

    const justifyContentValue = isRow
      ? isLeftAlignment
        ? 'flex-start'
        : isRightAlignment
        ? 'flex-end'
        : 'center'
      : 'flex-start'

    return (
      <Flex
        flexDirection={content.orientation}
        maxWidth="1250px"
        width="100%"
        alignItems={alignItemsValue}
        justifyContent={justifyContentValue}
        flexWrap="wrap"
        columnGap="5.25rem"
        rowGap="3.5rem"
        mb="2rem"
      >
        {content.components.map((component, index) => (
          <React.Fragment key={index}>
            {renderComponent(component)}
          </React.Fragment>
        ))}
      </Flex>
    )
  },
  componentcontaineritem: (content) => {
    const isColumn = content.orientation === 'column'
    const isRow = content.orientation === 'row'
    const isLeftAlignment = content.alignment === 'start'
    const isRightAlignment = content.alignment === 'end'

    const widthValue =
      content.width === 'Narrow'
        ? '350px'
        : content.width === 'Moderate'
        ? '500px'
        : content.width === 'Large'
        ? '715px'
        : '100%'

    const alignItemsValue = isColumn
      ? isLeftAlignment
        ? 'flex-start'
        : isRightAlignment
        ? 'flex-end'
        : 'center'
      : 'flex-start'

    const justifyContentValue = isRow
      ? isLeftAlignment
        ? 'flex-start'
        : isRightAlignment
        ? 'flex-end'
        : 'center'
      : 'flex-start'

    return (
      <Flex
        flexDirection={content.orientation}
        maxWidth={widthValue}
        width="100%"
        alignItems={alignItemsValue}
        justifyContent={justifyContentValue}
        flexWrap="wrap"
        gap="3.5rem"
        mb="2rem"
      >
        {content.components.map((component, index) => (
          <React.Fragment key={index}>
            {renderComponent(component)}
          </React.Fragment>
        ))}
      </Flex>
    )
  },
  componentimage: (content) => (
    <Image
      src={'https:' + content.src.fields.file.url}
      alt={content.description}
    />
  ),
  containerhighlight: (content) => (
    <>
      {content.typeOfSection === 'Regular' && (
        <Section
          pt="10rem"
          color={`brand.${content.textColour}`}
          backgroundColor={`brand.${content.backgroundColour}`}
        >
          <Container maxW="1250px" pb="150px">
            <Stack spacing="25px">
              {content.components.map((component, key) =>
                renderComponent(component, key)
              )}
            </Stack>
          </Container>
        </Section>
      )}
      {content.typeOfSection === 'Angular Right' && (
        <ExtendedSectionRight
          color={`brand.${content.textColour}`}
          backgroundColor={`brand.${content.backgroundColour}`}
          pt="5rem"
        >
          <Container maxW="1250px" pb="250px">
            <Stack spacing="25px">
              {content.components.map((component, key) =>
                renderComponent(component, key)
              )}
            </Stack>
          </Container>
        </ExtendedSectionRight>
      )}
      {content.typeOfSection === 'Angular Left' && (
        <ExtendedSectionLeft
          color={`brand.${content.textColour}`}
          backgroundColor={`brand.${content.backgroundColour}`}
          pt="3rem"
        >
          <Container maxW="1250px">
            <Stack spacing="25px">
              {content.components.map((component, key) =>
                renderComponent(component, key)
              )}
            </Stack>
          </Container>
        </ExtendedSectionLeft>
      )}
      {content.typeOfSection === 'Slanted' && (
        <SlantedSectionTop
          color={`brand.${content.textColour}`}
          backgroundColor={`brand.${content.backgroundColour}`}
        >
          <Container maxW="1250px" pb="250px">
            <Stack spacing="25px">
              {content.components.map((component, key) =>
                renderComponent(component, key)
              )}
            </Stack>
          </Container>
        </SlantedSectionTop>
      )}
    </>
  ),
  componentpageheader: (content) => (
    <Hero
      heading={content.heading}
      backgroundColour={`brand.${content.heroBackgroundColour}`}
      textColour={`brand.${content.textColour}`}
      pageTitle={content.pageTitle}
      subHeading={content.subHeading}
      image={content.heroImage?.fields?.file?.url}
      imageAlt={content.imageAlt}
    />
  ),
  componenthomepagehero: (content) => (
    <HomepageHero
      heading={content.heading}
      subHeading={content.subHeading}
      introduction={content.introduction}
      callToAction={content?.callToActionButton}
      callToActionLink={`/${content?.callToActionLink.fields.slug}`}
      heroImage={'https:' + content.heroImage.fields.file.url}
      heroImageAltText={content.heroImage.fields.file.description}
      headingBackground={content.headingBackground.fields.file.url}
    />
  ),
  componentrichtext: (content) => {
    let context = 'light'
    return (
      <Box pt="2">
        {content &&
          documentToReactComponents(content.content, renderRichText(context))}
      </Box>
    )
  },
  componentfeature: (content) => (
    <Feature
      featureItems={content.components}
      textColor={`brand.${content.textColour}`}
    />
  ),
  componentbutton: (content) => {
    const contentType = content?.link.sys.contentType.sys.id
    const linkSlug = content?.link.fields.slug

    let slug

    if (contentType === 'pageSimple') {
      if (linkSlug === 'news') {
        slug = `/${linkSlug}`
      } else {
        slug = `news/${linkSlug}`
      }
    } else {
      slug = `/${linkSlug}`
    }

    return (
      <Button
        variant={content?.variant}
        size={content?.size}
        colorScheme={content?.colorScheme}
        href={slug}
        as="a"
        textDecoration="none"
      >
        {content?.buttonText}
      </Button>
    )
  },

  componentcallout: (content) => {
    return (
      <Callout
        backgroundColor={`brand.${content.backgroundColor}` || 'brand.darkBlue'}
        textColor={`brand.${content.textColor}` || 'brand.white.500'}
        type={content.type}
      >
        {content.components.map((component, key) => (
          <Box key={key}>{renderComponent(component)}</Box>
        ))}
      </Callout>
    )
  },

  componentcalltoaction: (content) => (
    <Container maxW="1250px" py="10rem">
      <CallToAction
        heading={content.heading}
        description={content.description}
        buttonText={content.buttonText}
        link={content.link}
      />
    </Container>
  ),
  componentstatistic: (content) => {
    return (
      <Box>
        <Statistic
          statisticOne={content.statisticOne}
          labelOne={content.labelOne}
          statisticTwo={content.statisticTwo}
          labelTwo={content.labelTwo}
          textColor={`brand.${content.textColor}`}
        />
      </Box>
    )
  },
  codeembed: (content) => (
    <Box pb="10" pt="5">
      <Embed code={content.embed} />
    </Box>
  ),
  componentbanner: (content) => (
    <Banner
      type={content.type}
      icon={
        content.type == 'alert' ? (
          <ChakraAwesome icon="triangle-exclamation" />
        ) : (
          <ChakraAwesome icon="circle-info" />
        )
      }
      text={content.text}
      link={content.link}
      linkText={content.linkText}
    />
  ),

  componentcard: (content) => {
    const cardProps = {
      title: content.label,
      image: content.cardImage?.fields?.file?.url
        ? 'http:' + content.cardImage?.fields?.file?.url
        : '',
      alt: content?.cardImage?.fields?.title,
      link: content.link,
      body: content.cardBody,
      type: content.type,
      backgroundColor: `brand.${content.backgroundColor}` || 'brand.darkBlue',
      color: `brand.${content.textColor}` || 'brand.white.500',
      height: '100%',
    }

    return (
      <>
        {content.cardExtended ? (
          <CardWithExtended {...cardProps} />
        ) : (
          <Card {...cardProps} />
        )}
      </>
    )
  },

  componentsupportussection: (content) => {
    const [email, setEmail] = useState('')
    const [state, setState] = useState('idle')
    const [errorMsg, setErrorMsg] = useState(null)

    const subscribe = async (e) => {
      e.preventDefault()
      setState('Loading')

      try {
        const response = await axios.post('/api/actions/subscribe', { email })
        setState('Success')
        setEmail('')
        return response
      } catch (e) {
        console.log(e.response.data.error)
        setErrorMsg(e.response.data.error)
        setState('Error')
      }
    }
    return (
      <Flex as="section" width="100%" flexWrap="wrap">
        <Flex
          flexBasis="750px"
          bg="brand.legacyTeal"
          flexGrow="1"
          alignItems="flex-start"
          justifyContent={{ base: 'center', xl: 'flex-end' }}
        >
          <Stack
            maxWidth="750px"
            spacing="1rem"
            paddingInlineStart="1rem"
            marginInlineStart="auto"
            py="4rem"
            pr="2rem"
          >
            <Text color="brand.white.500" fontSize={{ base: '2xs', xl: 'xl' }}>
              {content.callToActionText}
            </Text>
            <Button> {content.callToActionButton}</Button>
          </Stack>
        </Flex>

        <Flex
          flexBasis="500px"
          flexGrow="1"
          alignItems="flex-start"
          bg="brand.brightTeal"
          justifyContent="flex-start"
        >
          <Stack
            maxWidth="500px"
            spacing="1rem"
            marginInlineEnd="auto"
            paddingInlineEnd="1rem"
            py="4rem"
            pl="2rem"
          >
            <Text>{content.subscribeToNewsletterText}</Text>

            <Stack width="100%" as="form" spacing="25px">
              <FormControl>
                <FormLabel>Your email</FormLabel>
                <Input
                  type="email"
                  name="email"
                  value={email}
                  onChange={(e) => setEmail(e.target.value)}
                />
              </FormControl>
              <Button
                as="a"
                href="https://www.google.com"
                size="short"
                disabled={state === 'Loading'}
                type="submit"
                variant="outline"
                colorScheme="brand.darkBlue"
                onClick={subscribe}
              >
                Sign up
              </Button>
              {state === 'Error' && <Text>{errorMsg}</Text>}
              {state === 'Success' && <Text>You&apos;ve been subscribed!</Text>}
            </Stack>
          </Stack>
        </Flex>
      </Flex>
    )
  },

  componentdownloadcard: (content) => {
    const cardProps = {
      title: content.label,
      image: content.cardImage?.fields?.file?.url
        ? 'http:' + content.cardImage?.fields?.file?.url
        : '',
      height: '100%',
      alt: content?.cardImage?.fields?.title,
      downloadFileURL: 'https:' + content?.downloadFile?.fields?.file?.url,
      downloadFileTitle: content?.downloadFile?.fields?.title,
      body: content.cardBody,
      type: content.type,
      backgroundColor: `brand.${content.backgroundColor}` || 'brand.darkBlue',
      color: `brand.${content.textColor}` || 'brand.white.500',
    }

    return (
      <>
        {content.cardExtended ? (
          <CardWithExtended {...cardProps} />
        ) : (
          <Card {...cardProps} />
        )}
      </>
    )
  },

  page: (content) => {
    return (
      <Card
        title={content.title}
        image={content.seo.fields.image?.fields?.file?.url}
        link={content.slug}
      >
        <Text>{content.seo.fields.description}</Text>
      </Card>
    )
  },
  patternlatestnews: (component, articles) => {
    return (
      <LatestNews
        articles={articles}
        backgroundColor="brand.brightPink"
        cardBackgroundColor="brand.brightTeal"
        headingColor="brand.white.500"
        cardTextColor="brand.darkBlue"
        backgroundImage={'https:' + component.backgroundImage.fields.file.url}
      />
    )
  },
}

function renderComponent(component, articles, context = 'light') {
  const type = component.__typename || component.sys.contentType.sys.id
  const content = component.fields || component
  let id = component.sys.id

  try {
    let rendered = ContentTypes[type.toLowerCase()](
      content,
      articles,
      context,
      id
    )
    return rendered
  } catch (err) {
    console.log(`Can not render ${type}, ${err}`)
  }
}

export { renderComponent, ContentTypes }
